
#include "nunchuk.h"
#include "tm_stm32f4_i2c.h"
#include "Math.h"

#define MAX_JOY 123
#define MAX_ACC 65

/* private variables */
float AX = 0;
float AY = 0;
float AZ = 0;
int JX = 0;
int JY = 0;
int BC = 0;
int BZ = 0;

uint8_t data[] = {0, 1, 2, 3, 4, 5};

I2C_TypeDef* I2C;
uint8_t nunchuk_address;
/* fin private variables */


void nunchuk_Init(I2C_TypeDef* I2Cx, uint8_t address, TM_I2C_PinsPack_t pinsPack, uint32_t speed) {
	I2C = I2Cx;
	nunchuk_address = address;

	//Initialize I2C, SCL: PB6 and SDA: PB7 with 100kHt serial clock
	TM_I2C_Init(I2C, pinsPack, speed);

	TM_I2C_Start(I2C, nunchuk_address, I2C_TRANSMITTER_MODE, I2C_ACK_DISABLE);
	TM_I2C_WriteData(I2C, 0xF0);
	TM_I2C_WriteData(I2C, 0x55);
	TM_I2C_Stop(I2C);
	int i = 1000;
	while(i > 0) {
		i--;
	}
	TM_I2C_Start(I2C, nunchuk_address, I2C_TRANSMITTER_MODE, I2C_ACK_DISABLE);
	TM_I2C_WriteData(I2C, 0xFB);
	TM_I2C_WriteData(I2C, 0x01);
	//TM_I2C_WriteData(I2C, (uint8_t)0x00);
	TM_I2C_Stop(I2C);
}

void nunchuk_SendZero() {
	TM_I2C_Start(I2C, nunchuk_address, I2C_TRANSMITTER_MODE, I2C_ACK_DISABLE);
	TM_I2C_WriteData(I2C, (uint8_t)0x00);
	TM_I2C_Stop(I2C);
}

int nunchuk_Update() {

	TM_I2C_ReadMultiNoRegister(I2C, nunchuk_address, data, 6);

	//const uint8_t buf[] = {0};
	//I2C_Write(I2C1 , buf , 1, NUNCHUK_ADDRESS);
	//I2C_Read(I2C1 , data , 6, NUNCHUK_ADDRESS);

	int resultado = (//data[0] != 0x1 &&
					 data[1] != 0x1 && data[2] != 0x1 &&
					 data[3] != 0x1 && data[4] != 0x1
					 && data[5] != 0x1)? 1 : 0;

	//data = (data ^ 0x17) + 0x17;
	//uint8_t resto = (data[5] >> 2) & 0x03;
	//AX = ((float)(((uint16_t)data[2] << 2) | resto)) - ZEROX;
	AX = (float) ((data[2] << 2) + ((data[5] & (00000011 << 2)) >> 2)) - ZEROX;

	//resto = (data[5] >> 4)& 0x03;
	//AY = ((float)(((uint16_t)data[3] << 2) | resto)) - ZEROY;
	AY = (float) ((data[3] << 2) + ((data[5] & (00000011 << 4)) >> 4)) - ZEROY;

	//resto = (data[5] >> 6) & 0x03;;
	//AZ = ((float)(((uint16_t)data[4] << 2) | resto))  - ZEROZ;
	AZ = (float) ((data[4] << 2) + ((data[5] & (00000011 << 6)) >> 6)) - ZEROZ;

	JX = (int) data[0] - DEFAULT_ZERO_JOY_X;
	JY = (int) data[1] - DEFAULT_ZERO_JOY_Y;

	BC = abs((int)((data[5] >> 1) & 0x01) - 1);
	BZ = abs((int) (data[5] & 0x01) - 1);
	return resultado;
}

int normalize(int value, int max) {
	if (value > max) {
		return max;
	} else if (value < -max) {
		return -max;
	}
	return value;
}

int nunchuk_GetC() {
	return BC;
}

int nunchuk_GetZ() {
	return BZ;
}

int nunchuk_GetJoystickX() {
	return normalize(JX, MAX_JOY);
}

int nunchuk_GetJoystickY() {
	return normalize(JY, MAX_JOY);
}

int nunchuk_GetPitch() {		// -65 <-> 65
	//return (float) atan(AX/sqrt(pow(AY,2) + pow(AZ,2)));
	int pitch = (int) (acos(AY/RADIUS)/ M_PI * 180.0) - 65;
	return normalize(pitch, MAX_ACC);
}

int nunchuk_GetRoll() {			// -65 <-> 65
	//return (float) atan(AY/sqrt(pow(AX,2) + pow(AZ,2)));
	int roll = (int)(atan2(AX,AZ)/ M_PI * 180.0);
	return normalize(roll, MAX_ACC);
}

int nunchuk_GetAcelerometerX() {
	return AX;
}

int nunchuk_GetAcelerometerY() {
	return AY;
}

int nunchuk_GetAcelerometerZ() {
	return AZ;
}
