/**
 * @defgroup TM_I2C
 * @brief    I2C library for STM32F4xx - http://stm32f4-discovery.com/2014/05/library-09-i2c-for-stm32f4xx/
 * @{
 *
 *	\par Pinout
 *	
@verbatim
       |PINSPACK 1   |PINSPACK 2   |PINSPACK 3
I2CX   |SCL   SDA    |SCL   SDA    |SCL   SDA
       |             |             |
I2C1   |PB6   PB7    |PB8   PB9    |PB6   PB9
I2C2   |PB10  PB11   |PF1   PF0    |PH4   PH5
I2C3   |PA8   PC9    |PH7   PH8    |-     -
@endverbatim
 *
 * \par Custom pinout
*/
#include "stm32f4xx.h"
#include "tm_stm32f4_i2c.h"
#include "attributes.h"
#include "defines.h"
#include "tm_stm32f4_gpio.h"

/**
 * @defgroup TM_I2C_Macros
 * @brief    Library defines
 * @{
 */

/**
 * @brief  Timeout for I2C
 */
#define ZEROX 510
#define ZEROY 490
#define ZEROZ 460
#define RADIUS 210  // probably pretty universal

#define DEFAULT_ZERO_JOY_X 124
#define DEFAULT_ZERO_JOY_Y 132

void nunchuk_Init(I2C_TypeDef* I2Cx, uint8_t address, TM_I2C_PinsPack_t pinsPack, uint32_t speed);

void nunchuk_SendZero();

int nunchuk_Update();

int nunchuk_GetC();

int nunchuk_GetZ();

int nunchuk_GetJoystickX();

int nunchuk_GetJoystickY();

int nunchuk_GetPitch();

int nunchuk_GetRoll();

int nunchuk_GetAcelerometerX();

int nunchuk_GetAcelerometerY();

int getAcelerometerZ();
